from abc import ABCMeta, abstractmethod
import pandas as pd
from athena_pkg.AthenaQueryEngine import AthenaQueryEngine

class QueryEngineFactory:
    @staticmethod
    def get_query_engine(query_engine):
        if query_engine == 'ATHENA':
            return AthenaQueryEngine()
        # elif query_engine == 'ANOTHER_FUCKING_ENGINE':
        #     return AnotherFuckingQueryEngine()
        else:
            raise AssertionError('Sorry this query engine is not taken into account')
class IQueryEngine(metaclass=ABCMeta):
    @abstractmethod
    def connect(self, config):
        """Method that return the object it self"""
    @abstractmethod
    def get_calendars(self, columns):
        """Returns the whole DataFrame of the calendars
         data"""
    @abstractmethod
    def get_listings(self, columns):
        """Returns the whole DataFrame of the listings data"""
    @abstractmethod
    def number_of_available_nights(self, date):
        """Returns numbers of available nights for a specific date"""


import pandas as pd
# from QueryEngine import IQueryEngine

# def ConnectToAthena(aws_access_key_id,aws_secret_access_key,s3_staging_dir,region_name):
#     """Returns an instance to connect you to AWS Athena via pyAthena"""
#     from pyathena import connect
#     return connect(aws_access_key_id=aws_access_key_id,aws_secret_access_key=aws_secret_access_key,s3_staging_dir=s3_staging_dir,region_name=region_name)

# class AnotherFuckingQueryEngine(IQueryEngine):
#     def __init__(self):
#         super()
#     def connect(self, config):
#         return self
#     def get_calendars(self, column):
#         return self
#     def number_of_available_nights(self, date):
#         return self
# """
# Si un jour tu as besoin d'ajouter un autre QueryEngine tu le fais de la même façon que AthenaQueryEngine .. 
# """
# if __name__ == '__main__':
#     print('You want to use athena to query your data ... then use the factory to get it')
#     QueryEngineFactory.get_query_engine(query_engine='ATHENA').connect({}).number_of_available_nights(date='2020-06-06')
#     print('You want to use AnotherFuckingEngine ... then use the factory to get it')
#     QueryEngineFactory.\
#         get_query_engine(query_engine='ANOTHER_FUCKING_ENGINE')\
#         .connect({})\
#         .get_calendars('?????????')


from athena_pkg.QueryEngine import IQueryEngine
import pandas as pd
from pyathena import connect as athena_connect

class AthenaQueryEngine(IQueryEngine):
    def __init__(self):
        self.con = None
        super()
    def connect(self, config):
        self.con = athena_connect(
            aws_access_key_id=config.aws_access_key_id,
            aws_secret_access_key=config.aws_secret_access_key,
            s3_staging_dir=config.s3_staging_dir,
            region_name=config.region_name
        )
        return self
    def get_calendars(self, columns):
        return pd.read_sql("select " + columns + " from calendars", self.con)
    def get_listings(self,columns):
        return pd.read_sql("select " + columns + " from details", self.con)
    def number_of_available_nights(self, date):
        return pd.read_sql("select count(*) from calendars where date like'" + date + "'", self.con)['_col0'][0]